(*
 * ocamlsdl - Objective Caml interface for the SDL library
 * Copyright (C) 1999 Jean-Christophe FILLIATRE
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2, as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU Library General Public License version 2 for more details
 * (enclosed in the file LGPL).
 *)

(* $Id: sdl.ml,v 1.12 2002/12/09 22:02:50 filliatr Exp $ *)

exception SDL_failure of string

let _ = Callback.register_exception "SDL_failure" (SDL_failure "")


(* Initialization. *)

type init_flag =
  | TIMER
  | AUDIO
  | VIDEO
  | CDROM
  | JOYSTICK
  | NOPARACHUTE   (* Don't catch fatal signals *)
  | EVENTTHREAD   (* Not supported on all OS's *)
  | EVERYTHING

external init : init_flag list -> unit = "sdlstub_init"

external quit : unit -> unit = "sdlstub_quit"


(* Video. *)

module Video = struct


  type video_flag =
    | SWSURFACE   (* Surface is in system memory *)
    | HWSURFACE   (* Surface is in video memory *)
    | ANYFORMAT   (* Allow any video pixel format *)
    | HWPALETTE   (* Surface has exclusive palette *)
    | DOUBLEBUF   (* Set up double-buffered video mode *)
    | FULLSCREEN  (* Surface is a full screen display *)
    | HWACCEL     (* Blit uses hardware acceleration *)
    | SRCCOLORKEY (* Blit uses a source color key *)
    | RLEACCEL    (* Colorkey blit is RLE accelerated *)
    | SRCALPHA    (* Blit uses source alpha blending *)
    | SRCCLIPPING (* Blit uses source clipping *)
    | OPENGL

  type surface

  external free_surface : surface -> unit
    = "sdlstub_free_surface"

  external surface_width : surface -> int
    = "sdlstub_surface_width"
  external surface_height : surface -> int
    = "sdlstub_surface_height"
  external surface_flags : surface -> video_flag list
    = "sdlstub_surface_flags"
  external surface_bpp : surface -> int
    = "sdlstub_surface_bpp"
  external surface_rmask : surface -> int
    = "sdlstub_surface_rmask"
  external surface_gmask : surface -> int
    = "sdlstub_surface_gmask"
  external surface_bmask : surface -> int
    = "sdlstub_surface_bmask"
  external surface_amask : surface -> int
    = "sdlstub_surface_amask"

  external must_lock : surface -> bool 
    = "sdlstub_must_lock"
  external lock_surface : surface -> unit
    = "sdlstub_lock_surface"
  external unlock_surface : surface -> unit
    = "sdlstub_unlock_surface"

  external video_mode_ok : int -> int -> int -> video_flag list -> bool 
    = "sdlstub_video_mode_ok"

  external set_video_mode : int -> int -> int -> video_flag list -> surface
    = "sdlstub_set_video_mode"

  external alloc_surface : 
    video_flag list -> int -> int -> int -> int -> int -> int -> int -> surface
    = "sdlstub_alloc_surface_byte" "sdlstub_alloc_surface"

  external load_bmp : string -> surface
    = "sdlstub_load_bmp"
  external save_bmp : surface -> string -> unit
    = "sdlstub_save_bmp"
 
  external set_color_key : surface -> video_flag list -> int -> unit
    = "sdlstub_set_color_key"
  external set_alpha : surface -> int -> int -> unit
    = "sdlstub_set_alpha"
(*
  external set_clipping : surface -> int -> int -> int -> int -> unit
    = "sdlstub_set_clipping"
  let disable_clipping s = set_clipping s 0 0 0 0
*)

  external display_format : surface -> surface
    = "sdlstub_display_format"
  
  external map_rgb : surface -> int -> int -> int -> int
    = "sdlstub_map_rgb"

  type rect = {
    mutable rect_x : int;
    mutable rect_y : int;
    mutable rect_w : int;
    mutable rect_h : int }

  external fill_surface : surface -> int -> unit
    = "sdlstub_fill_surface"
  external fill_rect : surface -> rect -> int -> unit
    = "sdlstub_fill_rect"

  external update_surface : surface -> unit
    = "sdlstub_update_surface"
  external update_rect : surface -> int -> int -> int -> int -> unit
    = "sdlstub_update_rect"
  external update_rects : surface -> int -> rect array -> unit
    = "sdlstub_update_rects"

  external flip : surface -> unit
    = "sdlstub_flip"

  external blit_surface : surface -> rect option ->
                          surface -> rect option -> unit 
    = "sdlstub_blit_surface"

  type color = { 
    red : int;
    green : int;
    blue : int 
  }
		 
  external set_colors : surface -> color array -> int -> int -> bool
    = "sdlstub_set_colors"

  external show_cursor : bool -> unit
    = "sdlstub_show_cursor"

  type raw
  external raw_of_surface : surface -> raw = "sdlstub_raw_of_surface"

end


(* Open GL support *)

module GL = struct

  external swap_buffers : unit -> unit = "sdlstub_GL_swap_buffers"

  external load_bmp : string -> Video.surface = "sdlstub_GL_load_bmp"

end


(* Events. *)

module Event  = struct

  type keysym = { 
    scancode : int;
    key : int;
    modif : int;
    unicode : int
  }

  type keystate = Pressed | Released

  type keyboard = {
    key_state : keystate;
    key_sym : keysym;
  }

  type event = 
    | Active 
    | KeyDown of keyboard
    | KeyUp of keyboard
    | MouseMotion
    | MouseButtonDown
    | MouseButtonUp
    | JoyMotion
    | JoyButtonDown
    | JoyButtonUp
    | Quit
    | SysWm
    | UnknowEvent

  external poll_event : unit -> event option
    = "sdlstub_poll_event"

  external wait_event : unit -> event
    = "sdlstub_wait_event"

end


(* Timer. *)

module Timer = struct

  external get_ticks : unit -> int
    = "sdlstub_get_ticks"

  external delay : int -> unit
    = "sdlstub_delay"

end


(* Drawing. *)

module Draw = struct

  external put_pixel : Video.surface -> int -> int -> int -> unit
    = "sdldraw_put_pixel"

  external get_pixel : Video.surface -> int -> int -> int
    = "sdldraw_get_pixel"

end



(* low-level audio. See also High-level audio in mixer.mli *)

module Audio = struct

(*
  external open_audio : unit -> unit
      = "sdlstub_open_audio"
*)

  external close_audio : unit -> unit
      = "sdlstub_close_audio"

  external unlock_audio : unit -> unit
      = "sdlstub_unlock_audio"

end

(*i
module Ttf = struct

  open Video

  (* Define a new exception for TTF errors and register 
     it to be callable from C code. *)

  exception SDLttf_exception of string
  let _ = Callback.register_exception "SDLttf_exception" (SDLttf_exception "Any string")

  external init : unit -> unit = "sdlttf_stub_init"

  (* Types *)

  type font

  (* Native C external functions *)

  external open_font : string -> int -> font = "sdlttf_open_font"
  external close_font : font -> unit = "sdlttf_close_font"
  external font_height : font -> int = "sdlttf_font_height"
  external font_metrics : font -> int -> int = "sdlttf_font_metrics"
  external render_text : font -> string -> (int*int*int) -> (int*int*int) -> Video.surface = "sdlttf_render_text"

  let make_glyph font col str =
    let text = render_text font str col (0,0,0) in
    set_color_key text [Video.SRCCOLORKEY] (Draw.get_pixel text 0 0);
    let conv = display_format text in
    free_surface text;
    conv

  (* ML functions *)

  let rect (x,y,w,h) = Some { rect_x = x; rect_y = y; rect_w = w; rect_h = h }

  let make_printer font col =
    (* make a memo array *)
    let glyphs = Array.make 256 (make_glyph font col "a") in
    for i = 1 to 255 do
      glyphs.(i) <- make_glyph font col (String.make 1 (Char.chr i))
    done;
    let blaa = Array.make 256 0 in
    for i = 1 to 255 do
      blaa.(i) <- font_metrics font i
    done;
    let height = font_height font in
    let free_fonts () =
      for i = 0 to 255 do
	free_surface glyphs.(i)
      done in  
    let print_string screen x y str =
      let x = ref x in
      for i = 0 to String.length str - 1 do
	let code = Char.code str.[i] in
	let img = glyphs.(code) in
	let w = surface_width img
	and h = surface_height img
	and yplus = blaa.(code) in
	blit_surface img (rect (0,0,w,h)) screen (rect (!x, y-yplus,w,h));
	x := !x + w;
	if str.[i] = ' ' then x := !x + 5
      done in
    print_string, free_fonts

end
i*)
